import unittest
from unittest.mock import patch, Mock

from ovos_translate_server_plugin import OVOSTranslateServer, OVOSLangDetectServer


class TestOVOSLangDetectServer(unittest.TestCase):

    @patch('requests.get')
    def test_detect(self, mock_get):
        dt = OVOSLangDetectServer()

        # Mock response for the language detection request
        mock_response = Mock()
        mock_response.ok = True
        mock_response.json.return_value = ["en"]
        mock_get.return_value = mock_response

        # Test language detection
        text = "Hello, world!"
        detected_language = dt.detect(text)
        self.assertEqual(detected_language, "en")

    @patch('requests.get')
    def test_detect_probs(self, mock_get):
        dt = OVOSLangDetectServer()

        # Mock response for the language detection probabilities request
        mock_response = Mock()
        mock_response.ok = True
        mock_response.json.return_value = {"en": 0.95, "es": 0.05}
        mock_get.return_value = mock_response

        # Test language detection probabilities
        text = "Hello, world!"
        detected_probs = dt.detect_probs(text)
        self.assertEqual(detected_probs, {"en": 0.95, "es": 0.05})

    def test_get_servers(self):
        dt = OVOSLangDetectServer()

        # Test when host is specified
        dt.host = "https://custom.server"
        self.assertEqual(dt.get_servers(), ["https://custom.server"])

        # Test when host is not specified
        dt.host = None
        servers = dt.get_servers()
        self.assertIn("https://nllb.openvoiceos.org", servers)
        self.assertIn("https://translator.smartgic.io/nllb", servers)


class TestOVOSTranslateServer(unittest.TestCase):

    @patch('requests.get')
    def test_translate(self, mock_get):
        tx = OVOSTranslateServer()

        # Mock response for the translation request
        mock_response = Mock()
        mock_response.ok = True
        mock_response.text = "Hello, world!"
        mock_get.return_value = mock_response

        # Test translation
        text = "Hola Mundo"
        translated_text = tx.translate(text, target="en", source="es")
        self.assertEqual(translated_text, "Hello, world!")

    @patch('requests.get')
    def test_translate_without_source(self, mock_get):
        tx = OVOSTranslateServer()

        # Mock responses for the language detection and translation requests
        mock_response_detect = Mock()
        mock_response_detect.ok = True
        mock_response_detect.json.return_value = ["es"]
        mock_response_translate = Mock()
        mock_response_translate.ok = True
        mock_response_translate.text = "Hello, world!"
        mock_get.side_effect = [mock_response_detect, mock_response_translate]

        # Test translation without specifying source language
        text = "Hola Mundo"
        translated_text = tx.translate(text, target="en")
        self.assertEqual(translated_text, "Hello, world!")

    def test_get_servers(self):
        tx = OVOSTranslateServer()

        # Test when host is specified
        tx.host = "https://custom.server"
        self.assertEqual(tx.get_servers(), ["https://custom.server"])

        # Test when host is not specified
        tx.host = None
        servers = tx.get_servers()
        self.assertIn("https://nllb.openvoiceos.org", servers)
        self.assertIn("https://translator.smartgic.io/nllb", servers)
        self.assertIn("https://ovosnllb.ziggyai.online", servers)


if __name__ == "__main__":
    unittest.main()



import random
import requests
from ovos_plugin_manager.templates.language import LanguageDetector, LanguageTranslator
from ovos_utils import classproperty
from typing import Union, List, Dict, Optional, Set


class OVOSLangDetectServer(LanguageDetector):
    PUBLIC_MODEL = "ovos-lang-detector-fasttext-plugin"  # manually maintained, public servers need to respect this to get added to list
    public_servers = [
        "https://nllb.openvoiceos.org",
        "https://translator.smartgic.io/nllb",
        # "https://ovosnllb.ziggyai.online"  # TODO - not yet using fasttext, needs to update container
    ]

    def __init__(self, *args, **kwargs):
        """
        Initialize the language detection server.

        Args:
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.
        """
        super().__init__(*args, **kwargs)
        self.host: Optional[Union[str, List[str]]] = self.config.get("host", None)

    def detect(self, text: str) -> str:
        """
        Detect the language of the given text.

        Args:
            text (str): Text to detect the language for.

        Returns:
            str: Detected language code.
        """
        text = text.replace("/", "-")  # HACK - if text has a / the url is invalid
        for url in self.get_servers():
            try:
                # call lang detect endpoint
                r = requests.get(f'{url}/detect/{text}')
                if r.ok:
                    try:
                        return r.json()[0]
                    except:
                        return r.text
            except:
                continue
        raise RuntimeError("All OVOS Translate servers are down!")

    def detect_probs(self, text: str) -> Dict[str, float]:
        """
        Detect the language probabilities for the given text.

        Args:
            text (str): Text to detect the language probabilities for.

        Returns:
            Dict[str, float]: Dictionary of language codes and their probabilities.
        """
        text = text.replace("/", "-")  # HACK - if text has a / the url is invalid
        for url in self.get_servers():
            try:
                # call lang detect endpoint
                r = requests.get(f'{url}/classify/{text}')
                if r.ok:
                    return r.json()
            except:
                continue
        raise RuntimeError("All OVOS Translate servers are down!")

    def get_servers(self) -> List[str]:
        """
        Get the list of servers to use for language detection.

        Returns:
            List[str]: List of server URLs.
        """
        if self.host:
            if isinstance(self.host, str):
                servers = [self.host]
            else:
                servers = self.host
        else:
            servers = self.public_servers
            random.shuffle(servers)  # Spread the load among all public servers
        return servers

    @classproperty
    def available_languages(cls) -> Set[str]:
        """
        Get the available target languages with the service.

        Returns:
            Set[str]: A set of language codes.
        """
        return set()


class OVOSTranslateServer(LanguageTranslator):
    PUBLIC_MODEL = "ovos-translate-plugin-nllb"  # manually maintained, public servers need to respect this to get added to list
    public_servers = [
        "https://nllb.openvoiceos.org",
        "https://translator.smartgic.io/nllb",
        "https://ovosnllb.ziggyai.online"
    ]

    def __init__(self, *args, **kwargs):
        """
        Initialize the translation server.

        Args:
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.
        """
        super().__init__(*args, **kwargs)
        self.host: Optional[Union[str, List[str]]] = self.config.get("host", None)
        self.skip_detection: bool = self.config.get("skip_detection", False)

    def translate(self,
                  text: Union[str, List[str]],
                  target: str = "",
                  source: str = "") -> Union[str, List[str]]:
        """
        NLLB200 translate text(s) into the target language.

        Args:
            text (Union[str, List[str]]): Sentence(s) to translate.
            target (str, optional): Target language code. Defaults to "".
            source (str, optional): Source language code. Defaults to "".

        Returns:
            Union[str, List[str]]: Translation(s).
        """
        target = target or self.internal_language

        text = text.replace("/", "-")  # HACK - if text has a / the url is invalid
        for url in self.get_servers():
            try:
                if not source and not self.skip_detection:
                    # call lang detect endpoint
                    r = requests.get(f'{url}/detect/{text}')
                    try:
                        source = r.json()[0]
                    except:
                        source = r.text

                if source:
                    u = f'{url}/translate/{source}/{target}/{text}'
                else:
                    # let the server plugin determine source lang by itself
                    u = f'{url}/translate/{target}/{text}'

                r = requests.get(u)
                if r.ok:
                    return r.text
            except:
                continue
        raise RuntimeError("All OVOS Translate servers are down!")

    def get_servers(self) -> List[str]:
        """
        Get the list of servers to use for translation.

        Returns:
            List[str]: List of server URLs.
        """
        if self.host:
            if isinstance(self.host, str):
                servers = [self.host]
            else:
                servers = self.host
        else:
            servers = self.public_servers
            random.shuffle(servers)  # Spread the load among all public servers
        return servers

    @classproperty
    def available_languages(cls) -> Set[str]:
        """
        Get the available target languages with the service.

        Returns:
            Set[str]: A set of language codes.
        """
        return set()

    def supported_translations(self, source_lang: str) -> Set[str]:
        """
        Get the set of target languages to which the source language can be translated.

        Args:
            source_lang (Optional[str]): The source language code.

        Returns:
            Set[str]: A set of language codes that the source language can be translated to.
        """
        return self.available_languages


if __name__ == "__main__":
    dt = OVOSLangDetectServer()

    src = "es"
    tgt = "en-us"

    tx = OVOSTranslateServer()

    utts = "Hola Mundo"

    print("Detections: ", dt.detect_probs(utts))
    print("Translations:", tx.translate(utts, tgt, src))
    print("Translations:", tx.translate(utts, tgt))

    utts = "hello world"

    print("Detections: ", dt.detect_probs(utts))
    print("Translations:", tx.translate(utts, src, tgt))
